<?php

namespace Drupal\orejime\Form;

use Drupal\Component\Plugin\Factory\FactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Class OrejimeSettingsForm.
 *
 * @ingroup orejime
 */
class OrejimeSettingsForm extends ConfigFormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Creates a new OrejimeSettingsForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Component\Plugin\Factory\FactoryInterface $plugin_factory
   *   The condition plugin factory.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, FactoryInterface $plugin_factory, ModuleHandlerInterface $module_handler) {
    parent::__construct($config_factory);
    $this->moduleHandler = $module_handler;
    $this->condition = $plugin_factory->createInstance('request_path');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.condition'),
      $container->get('module_handler')
    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'orejime_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['orejime.settings'];
  }

  /**
   * Defines the settings form for Orejime entities.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   Form definition array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['orejime_settings'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Settings'),
    ];
    $form['account'] = [
      '#type' => 'details',
      '#group' => 'orejime_settings',
      '#title' => $this->t('General settings'),
    ];
    $form['ignore'] = [
      '#type' => 'details',
      '#group' => 'orejime_settings',
      '#title' => $this->t('Ignore settings'),
    ];
    $form['language'] = [
      '#type' => 'details',
      '#group' => 'orejime_settings',
      '#title' => $this->t('Language settings'),
    ];
    $form['account']['cookie_name'] = [
      '#title' => $this->t('Cookie Name'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('cookie_name', 'orejime'),
      '#required' => TRUE,
      '#description' => $this->t('You can customize the name of the cookie that Orejime uses for storing user consent decisions.'),
    ];
    $form['account']['expires_after_days'] = [
      '#title' => $this->t('Cookie expires after days'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('expires_after_days', '365'),
      '#required' => TRUE,
      '#description' => $this->t('You can set a custom expiration time for the Orejime cookie, in days.'),
    ];
    $form['account']['privacy_policy'] = [
      '#title' => $this->t('Privacy Policy'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('privacy_policy', 'orejime'),
      '#required' => TRUE,
      '#description' => $this->t('You must provide a link to your privacy policy page'),
    ];
    $form['account']['logo'] = [
      '#title' => $this->t('URL logo notice'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('logo'),
      '#required' => FALSE,
      '#description' => $this->t('Optional. You can pass an image url to show in the notice.'),
    ];
    $form['account']['must_consent'] = [
      '#title' => $this->t('Must consent'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('must_consent', FALSE),
      '#required' => FALSE,
      '#description' => $this->t('"mustConsent" is set to true, Orejime will directly display the consent manager modal and not allow the user to close it before having actively consented or declined the use of third-party apps.'),
    ];
    $form['account']['must_notice'] = [
      '#title' => $this->t('Must notice'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('must_notice', FALSE),
      '#required' => FALSE,
      '#description' => $this->t('If "mustNotice" is set to true, Orejime will display the consent notice and not allow the user to close it before having actively consented or declined the use of third-party apps. Has no effect if mustConsent is set to true.'),
    ];
    $form['account']['implicit_consent'] = [
      '#title' => $this->t('Implicit consent'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('implicit_consent', FALSE),
      '#required' => FALSE,
      '#description' => $this->t('If "implicitConsent" is set to true, Orejime will automatically accept cookies if the user continues his navigation on the website after the first page. If you enable this, you must warn the user of this behavior in the notice window. You can do that easily by overriding translation strings (see below).'),
    ];
    $form['account']['analytics'] = [
      '#title' => $this->t('UA codes to manage'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('analytics'),
      '#required' => FALSE,
      '#description' => $this->t('Liste here UA used in the website.'),
    ];
    $form['account']['orejime_css'] = [
      '#title' => $this->t('CSS Orejime'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('orejime_css'),
      '#required' => FALSE,
      '#description' => $this->t('Set here the url if you want a different CSS or use external link'),
    ];
    $form['account']['orejime_js'] = [
      '#title' => $this->t('JS Orejime'),
      '#type' => 'textfield',
      '#default_value' => $this->getSettings('orejime_js'),
      '#required' => FALSE,
      '#description' => $this->t('Set here the url if you want a different js for orejime or use external link'),
    ];
    $form['account']['debug'] = [
      '#title' => $this->t('Debug Mode'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSettings('debug'),
      '#required' => FALSE,
      '#description' => $this->t('Set Orejime in debug mode to have a few stuff logged in the console, like warning about missing translations.'),
    ];
    $data = $this->getSettings('request_path');
    if ($data) {
      $this->condition->setConfiguration($data);
    }
    $form['ignore'] += $this->condition->buildConfigurationForm($form['ignore'], $form_state);
    $form['language']['texts'] = [
      '#title' => $this->t('Texts of Orejime'),
      '#type' => 'textarea',
      '#rows' => 35,
      '#default_value' => $this->getSettings('texts'),
      '#required' => FALSE,
      '#description' => $this->t('Translation file. Example here : https://raw.githubusercontent.com/empreinte-digitale/orejime/master/src/translations/en.yml'),
    ];
    if ($this->moduleHandler->moduleExists('content_translation')) {
      $form_state->set(['content_translation', 'key'], 'language');
      $form['language'] += content_translation_enable_widget('orejime_service', 'orejime_service', $form, $form_state);
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * Implements_form_validate().
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    try {
      Yaml::parse($form_state->getValue('texts'));
    }
    catch (\Exception $e) {
      $form_state->setErrorByName('texts', $e->getMessage());
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * Allow to get settings or default value.
   *
   * @param string $name
   *   The name of settings.
   * @param string $default
   *   Give a default value.
   *
   * @return string
   *   The value of the settings or the default value if empty.
   */
  public function getSettings($name, $default = '') {
    $site_config = $this->config('orejime.settings');
    return $site_config->get($name) ?: $default;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->condition->submitConfigurationForm($form, $form_state);
    $this->config('orejime.settings')
      ->set('cookie_name', $form_state->getValue('cookie_name'))
      ->set('expires_after_days', $form_state->getValue('expires_after_days'))
      ->set('logo', $form_state->getValue('logo'))
      ->set('privacy_policy', $form_state->getValue('privacy_policy'))
      ->set('must_consent', $form_state->getValue('must_consent'))
      ->set('must_notice', $form_state->getValue('must_notice'))
      ->set('implicit_consent', $form_state->getValue('implicit_consent'))
      ->set('analytics', $form_state->getValue('analytics'))
      ->set('orejime_css', $form_state->getValue('orejime_css'))
      ->set('orejime_js', $form_state->getValue('orejime_js'))
      ->set('texts', $form_state->getValue('texts'))
      ->set('debug', $form_state->getValue('debug'))
      ->set('request_path', $this->condition->getConfiguration())
      ->save();
    parent::submitForm($form, $form_state);
  }

}
